compHistDists <-
function(DBA,method='MMD', CompIDs=NULL, Usefiltered=TRUE,
                           PeakIDs=NULL, NormMethod='DESeq',
                           overWrite=FALSE, HistField='PeakRawHists',run.parallel=TRUE,
                           verbose=2, save.file=TRUE, out.dir='.',sigma=NULL){

  if (!is.element('MD',names(DBA))){
    stop('No histograms provided, call GetPeakProfiles first')
  }



  ## cluster stuff
  if (run.parallel){
    NCPUs <- detectCores()
    logfile <- paste(out.dir,'cluster.log',sep='')
    cl <- makeCluster(NCPUs,outfile=logfile)
  }

  samples <- DBA$samples
  MD <- DBA$MD

  PEAKS <- MD[[HistField]]

  if (Usefiltered & is.element('Filter',names(MD))){
    message('Using filtered Peaks')
    filtids <- MD$Filter$FiltPeakIds
  } else{
    message('Using unfiltered Peaks')
    filtids <- seq(1,length(PEAKS))
  }
  if (is.null(PeakIDs)){
    PeakIDs <- filtids
  } else{
    PeakIDs <- intersect( PeakIDs,  filtids)
    message('Using subset of all Peaks.')
  }
  message('nPeaks = ',length(PeakIDs),' (of ',length(MD[[HistField]]),')')

  PEAKS <- PEAKS[PeakIDs]

  nPeaks <- length(PEAKS)


  if (is.null(CompIDs)){
    nSamples <- nrow(samples)
    nComps <- (nSamples^2-nSamples)/2
    CompIDs <- matrix(0,2,nComps)
    CompIDxs <- matrix(0,2,nComps)
    st <- 0
    for (j in 1:(nSamples-1)){
      CompIDs[1,st+1:(nSamples-j)] <- samples$SampleID[j]
      CompIDs[2,st+1:(nSamples-j)] <- samples$SampleID[(j+1):nSamples]

      CompIDxs[1,st+1:(nSamples-j)] <- j
      CompIDxs[2,st+1:(nSamples-j)] <- (j+1):nSamples

      st <- st+nSamples-j
    }

  } else {
    CompIDxs <- matrix(0,nrow(CompIDs),ncol(CompIDs))
    IDx1 <- sapply(CompIDs[1,],function(id){ix=which(samples$SampleID==id)})
    IDx2 <- sapply(CompIDs[2,],function(id){ix=which(samples$SampleID==id)})
    CompIDxs <- rbind(IDx1,IDx2)
    nComps <- ncol(CompIDs)
  }



  H1 <- samples$bamReads[CompIDxs[1,]]
  H1 <- lapply(H1,function(FN){fn=unlist(strsplit(FN,'/'));fn=fn[length(fn)]})
  H1 <- lapply(H1,function(FN){fn=unlist(strsplit(FN,'.bam'));fn=fn[1]})
  H2 <- samples$bamReads[CompIDxs[2,]]
  H2 <- lapply(H2,function(FN){fn=unlist(strsplit(FN,'/'));fn=fn[length(fn)]})
  H2 <- lapply(H2,function(FN){fn=unlist(strsplit(FN,'.bam'));fn=fn[1]})
  HistNames <- rbind(H1,H2)

  CompNames <- paste(CompIDs[1,],CompIDs[2,],sep=' vs ')

  if (is.null(NormMethod)){
    message('No normalization factors applied')
    Factors <- matrix(1,nrow(CompIDs),ncol(CompIDs))
  } else {
    NormFactors <- MD$NormFactors[[NormMethod]][[1]]
    #check if factor is available for all relevant samples
    if (!all( is.element(unique(unlist(HistNames)),names(NormFactors)))){
      tmp <- which(!is.element(unique(unlist(HistNames)),names(NormFactors)))
      message('no normalization factor found for sample: ',HistNames[tmp])
      stop('check normalization factors')
    }
    Factors <- matrix(0,nrow(CompIDs),ncol(CompIDs))
    Factors[1,] <- NormFactors[sapply(HistNames[1,],function(hn){idx=which(hn==names(NormFactors))})]
    Factors[2,] <- NormFactors[sapply(HistNames[2,],function(hn){idx=which(hn==names(NormFactors))})]
  }

 # check if DISTS already exist:
  TODO <- seq(1,ncol(CompIDs))
  if (any(names(MD)=='DISTS')){
    if (is.element(method,names(MD$DISTS))){

      DONE <- colnames(MD$DISTS[[method]])
      if (!overWrite){
        if (!identical(rownames(MD$DISTS[[method]]),names(PEAKS))){
          stop('different peak sets')}

        message(method,' ; ', DONE,' has already been computed')
        TODO <- which(!is.element(CompNames,DONE))
        CompNames <- CompNames[TODO]
        nComps <- length(TODO)
      } else{
        message('overwriting distances')}

    }
    else {

      message('appending new method ', method, ' to DISTS')
    }

  }

  if (nComps==0){
    return(DBA)}




  message('computing ', nComps ,' pairwise distances for ', nPeaks, ' Peaks' )
  message('may take a while...')


  if (run.parallel){
    cl.idx <-  clusterSplit(cl,1:nPeaks)
    PEAKSsplt <- lapply(cl.idx, function(i) PEAKS[i])
    v <- clusterApplyLB(cl,PEAKSsplt, compDistWrapper,CompIDxs,Factors,HistNames,method,sigma=sigma)
    #Dists <- compDistWrapper(PEAKSsplt[1],CompIDxs,Factors,HistNames,method,out.dir)
    stopCluster(cl)
    Dists <- matrix(NA,nPeaks,nComps)
    mDists <- matrix(NA,nPeaks,nComps)
    for (i in 1:length(v)){
      Dists[cl.idx[[i]],] <- v[[i]]$Dists
      if (method=='MMD'){
        mDists[cl.idx[[i]],] <- v[[i]]$mDists}
    }
  } else {
    D <- compDistWrapper(PEAKS,CompIDxs,Factors,HistNames,method,sigma=sigma)
    Dists <- D$Dists
    if (method=='MMD'){
      mDists <- D$mDists
    }
  }

  # if only subset of dist is calculated, set remaining distances to 'NA'
  if (nPeaks<length(MD[[HistField]])){
    D <-  matrix(NA,length(MD[[HistField]]),nComps)
    D[PeakIDs,] <- Dists
    Dists <- D
    if (method=='MMD'){
      D <-  matrix(NA,length(MD[[HistField]]),nComps)
      D[PeakIDs,] <- mDists
      mDists <- D
    }
    
  }

  rownames(Dists) <- names(MD[[HistField]])
  colnames(Dists) <- CompNames


 # check if DISTS already exist:
  if (any(names(MD)=='DISTS')){

    if (is.element(method,names(MD$DISTS))){

      if (!overWrite){
        message('appending distances')
        coln <- colnames(MD$DISTS[[method]])
        DISTS <- cbind(MD$DISTS[[method]], Dists)
        colnames(DISTS) <-  c(coln, colnames(Dists))
      } else{
        message('overwriting distances')
        DISTS <- Dists
      }
      MD$DISTS[[method]] <- DISTS
    } else {
      DISTS <- list(Dists)
      names(DISTS) <- method
      MD$DISTS <-c(MD$DISTS, DISTS)
    }

  } else {
    DISTS <- list(Dists)
    names(DISTS) <- method
    MD$DISTS <- DISTS
  }
  MD$mDISTS <- mDists

  
  DBA$MD <- MD
  if (save.file){
    out.file <- paste(out.dir, 'Dists.Rdata', sep='')
    message("writing to file", out.file, "\n")
    save(DBA,file = out.file)
  }
  return(DBA)

}
