detPeakPvals <-
function(DBA, method='MMD', group1, group2, name1='g1',
                         name2='g2', Usefiltered=TRUE,
                         PeakIDs=NULL, quantprobs=seq(0,1,0.05),
                         fieldName='NormTotalCounts', bNormWidth=FALSE,
                         bSampleMean=FALSE, overWrite=FALSE){

  samples <- DBA$samples
  MD <- DBA$MD
  DISTS <- MD$DISTS[[method]]
  Peaks <- dba.peakset(DBA,bRetrieve=TRUE)
  
  if (!is.element(fieldName,names(MD))){
    message(fieldName,' not found as element of MD, add element to MD first or change fieldName')
    stop('field missing')
  } else{
    message('using ',fieldName,' for pooling')
    Counts <- MD[[fieldName]]
  }
  if (bNormWidth){
    Counts <- Counts/(width(Peaks))
    message('Counts normalized by peak width')
  }
 
 if (Usefiltered && is.element('Filter',names(MD))){
    message('Using filtered Peaks')
    filtids <- MD$Filter$FiltPeakIds
  } else{
    message('Using unfiltered Peaks')
    filtids <- seq(1,nrow(Counts))
  }
  if (is.null(PeakIDs)){
    PeakIDs <- filtids
  } else{
    PeakIDs <- intersect( PeakIDs,  filtids)
    message('Using subset of all Peaks for normalization.')
  }
  message('nPeaks = ',length(PeakIDs),' (of ',nrow(Counts),')')
    
  compNames <- colnames(DISTS)

  within1 <- determineGroupComps(group1,type='within')
  within2 <- determineGroupComps(group2,type='within')
  between <- determineGroupComps(group1,group2,type='between')

  if (is.null(within1)&&is.null(within2)){
    stop('no replicates given to estimate within group variance')}

  ids1 <- findComps(within1,compNames,method)
  ids2 <- findComps(within2,compNames,method)
  ids3 <- findComps(between,compNames,method)
  ids <- c(ids1,ids2,ids3)
  na.ids <- which(is.na(DISTS[,ids]),arr.ind=TRUE)
  missingPeaks <- unique(na.ids[,'row'])
  PeakIDs <- setdiff(PeakIDs,missingPeaks)
  if (length(missingPeaks)>0){
    message('Distances missing for some Peaks')}
  
  message('nPeaks = ',length(PeakIDs),' (of ',nrow(Counts),')')
  
  W1 <- as.matrix(DISTS[PeakIDs,ids1])
  colnames(W1) <- colnames(DISTS)[ids1]
  W2 <- as.matrix(DISTS[PeakIDs,ids2])
  colnames(W2) <- colnames(DISTS)[ids2]
  B <- as.matrix(DISTS[PeakIDs,ids3])
  colnames(B) <- colnames(DISTS)[ids3]

  
  temp <- c(within1,within2,between)
  temp <- unique(unlist(strsplit(temp,' vs ')))
  SampleIDs <- which(is.element(samples$SampleID,temp))

  #Counts <- log2(Counts[PeakIDs,SampleIDs])
  Counts <- Counts[PeakIDs,SampleIDs]
  
  #check  colnames(Counts)   

  if (bSampleMean){
    Ns <- as.matrix(rowMeans(Counts),ncol=1,nrow=nrow(Counts))
  } else {
    Ns <- Counts
    colnames(Ns)=samples$SampleID[SampleIDs]
  }
  
  dN <- quantile(Ns,probs <- quantprobs)
  message("dN:")
  print(round(dN))
  message("Computing p-values \n");
  
  if (method=='MMD'||method=='GMD'){
    P <- compute.pvalues(B,W1,W2,Ns,dN,Sign=FALSE)
  } else if (method=='Pearson') {
    P <- compute.pvalues(B,W1,W2,Ns,dN,Sign=FALSE,smaller=TRUE)
  }
  p.vals <- matrix(NA,nrow=nrow(DISTS),ncol=ncol(P[['pval']]))
  p.vals[PeakIDs,] <- P[['pval']]
  fdr <- matrix(NA,nrow=nrow(DISTS),ncol=ncol(P[['FDR']]))
  fdr[PeakIDs,] <- P[['FDR']]

  rownames(p.vals) <- rownames(DISTS)
  rownames(fdr) <- rownames(DISTS)
  colnames(p.vals) <- colnames(P[['pval']])
  colnames(fdr) <- colnames(P[['FDR']])
 
  
  
  # check if Pvals already exist:
  if (any(names(MD)=='Pvals')){

    if (any(names(MD$Pvals)==method)){

      if (!overWrite){
        message('appending p values')
        coln <- colnames(MD$Pvals[[method]])
        PVAL <- cbind(MD$Pvals[[method]], p.vals)
        colnames(PVAL) <-  c(coln, colnames(p.vals))
      } else{
        message('overwriting p values')
        PVAL <- p.vals
      }
      MD$Pvals[[method]] <- PVAL
      
    } else{
      Pvals <- list(p.vals)
      names(Pvals) <- method
      MD$Pvals <- c(MD$Pvals, Pvals)
      
    }

  } else {
    Pvals <- list(p.vals)
    names(Pvals) <- method
    MD$Pvals <- Pvals
  }
 # check if FDR already exist:
  if (any(names(MD)=='FDR')){

    if (any(names(MD$FDR)==method)){

      if (!overWrite){
        message('appending p values')
        coln <- colnames(MD$FDR[[method]])
        FDR <- cbind(MD$FDR[[method]], fdr)
        colnames(FDR) <-  c(coln, colnames(fdr))
      } else{
        message('overwriting p values')
        FDR <- fdr
      }
      MD$FDR[[method]] <- FDR
      
    } else{
      FDR <- list(fdr)
      names(FDR) <- method
      MD$FDR <- c(MD$FDR, FDR)
      
    }

  } else {
    FDR <- list(fdr)
    names(FDR) <- method
    MD$FDR <- FDR
  }

  
  DBA$MD <- MD
  
  return(DBA)
}
